<?php

namespace Drupal\headline_group\Plugin\Field\FieldWidget;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\headline_group\HeadlineGroupItemInterface;

/**
 * Plugin implementation of the 'headline_complete' widget.
 *
 * @FieldWidget(
 *   id = "headline_complete",
 *   label = "Headline Group (all fields)",
 *   field_types = {
 *     "headline_group"
 *   }
 * )
 */
class HeadlineCompleteWidget extends WidgetBase implements ContainerFactoryPluginInterface {

  /**
   * {@inheritdoc}
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings, EntityTypeManagerInterface $entity_type_manager) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->entity_type_manager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static($plugin_id, $plugin_definition, $configuration['field_definition'], $configuration['settings'], $configuration['third_party_settings'], $container
      ->get('entity_type.manager'));
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    if ($this->supportsSuperhead()) {
      $element['superhead'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Superhead'),
        '#placeholder' => NULL,
        '#default_value' => isset($items[$delta]->superhead) ? $items[$delta]->superhead : NULL,
        '#maxlength' => 255,
      ];
    }

    switch ($this->titleBehavior()) {
      case HeadlineGroupItemInterface::HG_OVERRIDE:
      case HeadlineGroupItemInterface::HG_PROHIBIT:
        // Get the label on the parent entity (used for placeholders below).
        $currentTitle = $items->getParent()->getValue()->label();

      case HeadlineGroupItemInterface::HG_OVERRIDE:
        $headline_placeholder = (empty($currentTitle)) ? $this->t('The main headline') : $currentTitle;
        $headline_disabled = FALSE;
        $headline_description = "If you do not provide a headline, the title will be used instead.";
        break;

      case HeadlineGroupItemInterface::HG_PROHIBIT:
        $headline_placeholder = (empty($currentTitle)) ? $this->t('Use the entity title.') : $currentTitle;
        $headline_disabled = TRUE;
        $headline_description = "The headline is automatically provided by the title.";
        break;

      default:
        // Inc. HG_BLANK.
        $headline_placeholder = $this->t('The main headline');
        $headline_disabled = FALSE;
        $headline_description = NULL;
    }

    $element['headline'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Headline'),
      '#placeholder' => $headline_placeholder,
      '#default_value' => isset($items[$delta]->superhead) ? $items[$delta]->headline : NULL,
      '#maxlength' => 255,
      '#disabled' => $headline_disabled,
      '#description' => $headline_description,
    ];

    if ($this->supportsSubhead()) {
      $element['subhead'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Subhead'),
        '#placeholder' => NULL,
        '#default_value' => isset($items[$delta]->subhead) ? $items[$delta]->subhead : NULL,
        '#maxlength' => 255,
      ];
    }

    $element += [
      '#type' => 'fieldset',
    ];

    return $element;

  }

  /**
   * Modify headline based on configuration.
   */
  public function massageFormValues(array $values, array $form, FormStateInterface $form_state) {
    $title_behavior = $this->titleBehavior();
    if ($title_behavior == HeadlineGroupItemInterface::HG_BLANK) {
      // no special title handling
      return $values;
    }
    $parent_title = $this->getParentTitle($form, $form_state);
    
    foreach ($values as $delta => $val) {
      switch ($title_behavior) {
        case HeadlineGroupItemInterface::HG_OVERRIDE:
          if (empty($values[$delta]['headline'])) {
            $values[$delta]['headline'] = $parent_title;
          }
          break;

        case HeadlineGroupItemInterface::HG_PROHIBIT:
          $values[$delta]['headline'] = $parent_title;
          break;

      }
    }
    return $values;
  }

  /**
   * Indicates enabled support for superheads.
   *
   * @return bool
   *   Returns TRUE if the HeadlineGroupItem field is configured to support
   *   superheads, otherwise FALSE.
   */
  protected function supportsSuperhead() {
    $support = $this->getFieldSetting('include_superhead');
    return (bool) (HeadlineGroupItemInterface::HG_SUPERHEAD === $support);
  }

  /**
   * Indicates enabled support for subheads.
   *
   * @return bool
   *   Returns TRUE if the HeadlineGroupItem field is configured to support
   *   subheads, otherwise FALSE.
   */
  protected function supportsSubhead() {
    $support = $this->getFieldSetting('include_subhead');
    return (bool) (HeadlineGroupItemInterface::HG_SUBHEAD === $support);
  }

  /**
   * Returns the preference for headline treatment.
   */
  protected function titleBehavior() {
    return $this->getFieldSetting('title_behavior');
  }

  /**
   * Figure out the title field of the parent entity and return it if available.
   */
  protected function getParentTitle(array $form, FormStateInterface $form_state) {
    $label_field = FALSE;
    $title = '';

    if (isset($form['#entity_type'])) {
      // This is an IEF subform -- look up the keys for the entity.
      $entity_type = $form['#entity_type'];
      if ($this->entity_type_manager->getDefinition($entity_type)->hasKey('label')) {
        $label_field = $this->entity_type_manager->getDefinition($entity_type)->getKey('label');
      }
    }
    else {
      // Get the label field from the form object.
      $entity = $form_state->getFormObject()->getEntity();
      if ($entity->getEntityType()->hasKey('label')) {
        $label_field = $entity->getEntityType()->getKey('label');
      }
    }

    if ($label_field) {
      // Get the current title from the form state.
      if ($label_field) {
        $title_vals = $form_state->getValue($label_field);
        if (isset($title_vals[0]['value'])) {
          $title = $title_vals[0]['value'];
        }
      }
    }

    return $title;
  }

}
